/*
 * PM - 2017
 * lab0.c
 */
#include <stdio.h>
 
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "lcd.h"

#include "usart.h"
#include "pff.h"

#define USART_BUFFER_SIZE 35
char usart_buf[USART_BUFFER_SIZE];

/* Led de 1W */
#define POWER_LED PB0

/* LED-uri indicatoare */
#define USER_LED PD7
#define ELECTROVALVE_LED PC1
#define SOL_LED_0 PC2
#define SOL_LED_1 PC3
#define LUMIN_LED PC4
#define TEMP_LED PC5

#define BAUD 9600
#define BAUDRATE ((F_CPU)/(BAUD*16UL)-1)
/* ADC stuff */
#define ADC_INIT_CHANNEL 0
#define ADC_AREF_VOLTAGE 5
#define ADC_MAX_CHANNELS 4

#define LIGHT_CH 0
#define UM_SOL1_CH 1
#define UM_SOL2_CH 2
#define TEMP_CH 3
/* SD card stuff */
#define RESULTS "results"

/* Numele canalelor de ADC. */
typedef struct
{
	const char *name;
	uint8_t channel;
	uint8_t	indicator;
	const char *path;
} adc_channel_t;


/* Perechi de (Nume canal, Numar canal). */
const adc_channel_t ADC_channels[] = {
	{"Lumin", LIGHT_CH, LUMIN_LED, "luminozitate"},
	{"Um_sol_0", UM_SOL1_CH, SOL_LED_0, "umiditate_sol_0"},
	{"Um_sol_1", UM_SOL2_CH, SOL_LED_1, "umiditate_sol_1"},
	{"Temp", TEMP_CH, TEMP_LED, "temperatura"}
};

const int adc_num_channels = sizeof(ADC_channels) /
	sizeof(ADC_channels[0]);
	
/* Store ADC values */
volatile uint16_t ADC_value[ADC_MAX_CHANNELS];

ISR(ADC_vect) {

    static uint8_t channel = ADC_INIT_CHANNEL;
	
	/* store new result */
    ADC_value[channel] = ADC;
	
    /* change channel */
    channel = (channel + 1) % ADC_MAX_CHANNELS;

    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;
	
	/* start a new conversion if there are any channels left */
    if (channel != ADC_INIT_CHANNEL)
        ADCSRA |= (1 << ADSC);
}

ISR(TIMER1_COMPA_vect) {
    /* start an ADC conversion */
    ADCSRA |= (1 << ADSC);
}	
	
void ADC_init() {
	ADMUX = (1 << REFS0) |(1 << ADC_INIT_CHANNEL);
	ADCSRA = (1 << ADEN) | (1 << ADIE) | (5 << ADPS0);
}

double ADC_voltage(int raw)
{
    (void)raw;
    return raw * ADC_AREF_VOLTAGE / 1023.0;
}

uint16_t ADC_get(uint8_t channel)
{
    return ADC_value[channel];
}
void TIMER1_init()
{
    /* initialize timer to trigger COMPA every 100ms */
    TCCR1B = (5 << CS10) | (1 << WGM12);
    TIMSK1 |= (1 << OCIE1A);
    OCR1A = 1561;
}


void write_adc_to_file(const char *path, const adc_channel_t *channel) {
	
	FRESULT ret;
	char str[20];
	double adc_value;
	WORD bytes_write;

	ret = pf_open(path);
	
	if(ret != FR_OK)
	{
		sprintf(usart_buf, "File %s missing\n", path);
		USART0_print(usart_buf);
		return;
	}
	
	adc_value = ADC_voltage(ADC_get(channel->channel));
	sprintf(str, "%s:%0.2f\n", channel->name, adc_value);
	
	ret = pf_write(str, sizeof(str), &bytes_write);
	if (ret != FR_OK) {
		USART0_print("write error\n");
	}
	
}
#define WAIT_READS 25
uint16_t adc_reads = 0;
void ADC_show() {
	
	if (adc_reads == WAIT_READS) {
		adc_reads = 0;
	} else {
		adc_reads++;
	}
	sprintf(usart_buf, "Read: %d\n", adc_reads);
	USART0_print(usart_buf);
	for (int i = 0; i < adc_num_channels; i++) {
		const adc_channel_t *channel = &ADC_channels[i];
		double adc_value = ADC_voltage(ADC_get(channel->channel));
		char str[20];
		sprintf(str, "D:%s:%0.2f\n", channel->name, adc_value);
		USART0_print(str);
		if (adc_reads == WAIT_READS) {
			sprintf(usart_buf, "Write in file\n");
			USART0_print(usart_buf);
			write_adc_to_file(channel->path, channel);
		}
	}
	

}
void interpret_light() {
	const adc_channel_t *channel = &ADC_channels[LIGHT_CH];
	double adc_value = ADC_voltage(ADC_get(channel->channel));
	
	if (adc_value < 1.00) {
		PORTB |= _BV(POWER_LED);
	} else {
		PORTB &= ~_BV(POWER_LED);
	}
	
}


void init () {
	DDRD |= _BV(USER_LED);
	DDRB |= _BV(POWER_LED);
	
	//LCD_init();
	
	ADC_init();
	TIMER1_init();
	USART0_init(BAUDRATE);
	USART0_print("Init done\n");
	sei();

}

void blink_led(char led) {
	PORTD |= _BV(led);
	_delay_ms(500);
	PORTD &= ~_BV(led);
	_delay_ms(500);
}
int main(void) {
	
	init();
	
	while (1) {
		
		blink_led(USER_LED);
		ADC_show();
		interpret_light();
		USART0_print("*******************\n");
	}

	return 0;
}
